@@ -1,7 +1,7 @@
 /*
 	parse: spawned from common; clustering around stream/frame parsing
 
-	copyright ?-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2009 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp & Thomas Orgis
 */
@@ -63,7 +63,6 @@
 	(still unsure about this private bit)
 */
 #define HDRCMPMASK 0xfffe0d00
-#define HDRCHANMASK 0xc0  /*      11000000, selecting II bits (channel mode) */
 #define HDRSAMPMASK 0xc00 /* 1100 00000000, FF bits (sample rate) */
 
 /* bitrates for [mpeg1/2][layer] */
@@ -119,9 +118,6 @@ int head_check(unsigned long head)
 		/* 1111 means bad bitrate */
 		(((head>>12)&0xf) == 0xf)
 		||
-		/* 0000 means free format... which should be supported in future. */
-		(((head>>12)&0xf) == 0x0)
-		||
 		/* sampling freq: 11 is reserved */
 		(((head>>10)&0x3) == 0x3 )
 		/* here used to be a mpeg 2.5 check... re-enabled 2.5 decoding due to lack of evidence that it is really not good */
@@ -147,7 +143,7 @@ static int check_lame_tag(mpg123_handle *fr)
 		Also, how to avoid false positives? I guess I should interpret more of the header to rule that out(?).
 		I hope that ensuring all zeros until tag start is enough.
 	*/
-	size_t lame_offset = (fr->stereo == 2) ? (fr->lsf ? 17 : 32 ) : (fr->lsf ? 9 : 17);
+	int lame_offset = (fr->stereo == 2) ? (fr->lsf ? 17 : 32 ) : (fr->lsf ? 9 : 17);
 	/* At least skip the decoder delay. */
 #ifdef GAPLESS
 	if(fr->begin_s == 0) frame_gapless_init(fr, GAPLESS_DELAY, 0);
@@ -155,7 +151,7 @@ static int check_lame_tag(mpg123_handle *fr)
 
 	if(fr->framesize >= 120+lame_offset) /* traditional Xing header is 120 bytes */
 	{
-		size_t i;
+		int i;
 		int lame_type = 0;
 		debug("do we have lame tag?");
 		/* only search for tag when all zero before it (apart from checksum) */
@@ -224,15 +220,33 @@ static int check_lame_tag(mpg123_handle *fr)
 				}
 				if(xing_flags & 0x2) /* bytes */
 				{
-					if(VERBOSE3)
+					unsigned long xing_bytes = make_long(fr->bsbuf, lame_offset);					/* We assume that this is the _total_ size of the file, including Xing frame ... and ID3 frames...
+					   It's not that clearly documented... */
+					if(fr->rdat.filelen < 1)
+					fr->rdat.filelen = (off_t) xing_bytes; /* One could start caring for overflow here. */
+					else
 					{
-						unsigned long xing_bytes = make_long(fr->bsbuf, lame_offset);
-						fprintf(stderr, "Note: Xing: %lu bytes\n", (long unsigned)xing_bytes);
+						if((off_t) xing_bytes != fr->rdat.filelen && NOQUIET)
+						{
+							double diff = 1.0/fr->rdat.filelen * (fr->rdat.filelen - (off_t)xing_bytes);
+							if(diff < 0.) diff = -diff;
+
+							if(VERBOSE3)
+							fprintf(stderr, "Note: Xing stream size %lu differs by %f%% from determined/given file size!\n", xing_bytes, diff);
+
+							if(diff > 1.)
+							fprintf(stderr, "Warning: Xing stream size off by more than 1%%, fuzzy seeking may be even more fuzzy than by design!\n");
+						}
 					}
+
+					if(VERBOSE3)
+					fprintf(stderr, "Note: Xing: %lu bytes\n", (long unsigned)xing_bytes);
+
 					lame_offset += 4;
 				}
 				if(xing_flags & 0x4) /* TOC */
 				{
+					frame_fill_toc(fr, fr->bsbuf+lame_offset);
 					lame_offset += 100; /* just skip */
 				}
 				if(xing_flags & 0x8) /* VBR quality */
@@ -319,7 +333,7 @@ static int check_lame_tag(mpg123_handle *fr)
 							else if(gt == 2) gt = 1; /* audiophile */
 							else continue;
 							/* get the 9 bits into a number, divide by 10, multiply sign... happy bit banging */
-							replay_gain[0] = ((fr->bsbuf[lame_offset] & 0x2) ? -0.1 : 0.1) * (make_short(fr->bsbuf, lame_offset) & 0x1f);
+							replay_gain[0] = (float) ((fr->bsbuf[lame_offset] & 0x2) ? -0.1 : 0.1) * (make_short(fr->bsbuf, lame_offset) & 0x1f);
 						}
 						lame_offset += 2;
 					}
@@ -372,6 +386,11 @@ static int check_lame_tag(mpg123_handle *fr)
 	return 0; /* no lame tag */
 }
 
+/* Just tell if the header is some mono. */
+static int header_mono(unsigned long newhead)
+{
+	return ((newhead>>6)&0x3) == MPG_MD_MONO ? TRUE : FALSE;
+}
 
 /*
 	That's a big one: read the next frame. 1 is success, <= 0 is some error
@@ -388,7 +407,7 @@ int read_frame(mpg123_handle *fr)
 	int oldphase = fr->halfphase;
 	fr->fsizeold=fr->framesize;       /* for Layer3 */
 
-	/* Hm, I never tested this...*/
+	/* Speed-down hack: Play it again, Sam (the frame, I mean). */
 	if (fr->p.halfspeed) 
 	{
 		if(fr->halfphase) /* repeat last frame */
@@ -417,22 +436,21 @@ init_resync:
 	fr->header_change = 2; /* output format change is possible... */
 	if(fr->oldhead)        /* check a following header for change */
 	{
+		if(fr->oldhead == newhead) fr->header_change = 0;
+		else
 		/* If they have the same sample rate. Note that only is _not_ the case for the first header, as we enforce sample rate match for following frames.
 			 So, during one stream, only change of stereoness is possible and indicated by header_change == 2. */
 		if((fr->oldhead & HDRSAMPMASK) == (newhead & HDRSAMPMASK))
 		{
-			/* Now if both channel modes are mono... */
-			if( (fr->oldhead & HDRCHANMASK) == 0 && (newhead & HDRCHANMASK) == 0)
-			fr->header_change = 1;
-			/* ...or stereo (of sorts), then we have a small header change */
-			else if( (fr->oldhead & HDRCHANMASK) > 0 && (newhead & HDRCHANMASK) > 0)
+			/* Now if both channel modes are mono or both stereo, it's no big deal. */
+			if( header_mono(fr->oldhead) == header_mono(newhead))
 			fr->header_change = 1;
 		}
 	}
 
 #ifdef SKIP_JUNK
 	/* watch out for junk/tags on beginning of stream by invalid header */
-	if(!fr->firsthead && !head_check(newhead) && !free_format_header(newhead)) {
+	if(!fr->firsthead && !head_check(newhead)) {
 		int i;
 
 		/* check for id3v2; first three bytes (of 4) are "ID3" */
@@ -441,24 +459,30 @@ init_resync:
 			int id3ret = 0;
 			id3ret = parse_new_id3(fr, newhead);
 			if     (id3ret < 0){ debug("need more?"); ret = id3ret; goto read_frame_bad; }
+#ifndef NO_ID3V2
 			else if(id3ret > 0){ debug("got ID3v2"); fr->metaflags  |= MPG123_NEW_ID3|MPG123_ID3; }
 			else debug("no useful ID3v2");
+#endif
 
 			fr->oldhead = 0;
 			goto read_again; /* Also in case of invalid ID3 tag (ret==0), try to get on track again. */
 		}
-		else if(VERBOSE2) fprintf(stderr,"Note: Junk at the beginning (0x%08lx)\n",newhead);
+		else if(VERBOSE2 && fr->silent_resync == 0) fprintf(stderr,"Note: Junk at the beginning (0x%08lx)\n",newhead);
 
 		/* I even saw RIFF headers at the beginning of MPEG streams ;( */
 		if(newhead == ('R'<<24)+('I'<<16)+('F'<<8)+'F') {
-			if(VERBOSE2) fprintf(stderr, "Note: Looks like a RIFF header.\n");
+			if(VERBOSE2 && fr->silent_resync == 0) fprintf(stderr, "Note: Looks like a RIFF header.\n");
+
 			if((ret=fr->rd->head_read(fr,&newhead))<=0){ debug("need more?"); goto read_frame_bad; }
+
 			while(newhead != ('d'<<24)+('a'<<16)+('t'<<8)+'a')
 			{
 				if((ret=fr->rd->head_shift(fr,&newhead))<=0){ debug("need more?"); goto read_frame_bad; }
 			}
 			if((ret=fr->rd->head_read(fr,&newhead))<=0){ debug("need more?"); goto read_frame_bad; }
-			if(VERBOSE2) fprintf(stderr,"Note: Skipped RIFF header!\n");
+
+			if(VERBOSE2 && fr->silent_resync == 0) fprintf(stderr,"Note: Skipped RIFF header!\n");
+
 			fr->oldhead = 0;
 			goto read_again;
 		}
@@ -535,13 +559,6 @@ init_resync:
 	/* why has this head check been avoided here before? */
 	if(!head_check(newhead))
 	{
-		if(!fr->firsthead && free_format_header(newhead))
-		{
-			if(NOQUIET) error1("Header 0x%08lx seems to indicate a free format stream; I do not handle that yet", newhead);
-
-			goto read_again;
-			return 0;
-		}
 		/* and those ugly ID3 tags */
 		if((newhead & 0xffffff00) == ('T'<<24)+('A'<<16)+('G'<<8))
 		{
@@ -566,7 +583,7 @@ init_resync:
 			fr->metaflags  |= MPG123_NEW_ID3|MPG123_ID3;
 			goto read_again;
 		}
-		else if(NOQUIET)
+		else if(NOQUIET && fr->silent_resync == 0)
 		{
 			fprintf(stderr,"Note: Illegal Audio-MPEG-Header 0x%08lx at offset 0x%lx.\n",
 				newhead, (long unsigned int)fr->rd->tell(fr)-4);
@@ -579,8 +596,12 @@ init_resync:
 		{
 			long try = 0;
 			long limit = fr->p.resync_limit;
+			
+			/* If a resync is needed the bitreservoir of previous frames is no longer valid */
+			fr->bitreservoir = 0;
+
 			/* TODO: make this more robust, I'd like to cat two mp3 fragments together (in a dirty way) and still have mpg123 beign able to decode all it somehow. */
-			if(NOQUIET) fprintf(stderr, "Note: Trying to resync...\n");
+			if(NOQUIET && fr->silent_resync == 0) fprintf(stderr, "Note: Trying to resync...\n");
 			/* Read more bytes until we find something that looks
 			 reasonably like a valid header.  This is not a
 			 perfect strategy, but it should get us back on the
@@ -589,16 +610,17 @@ init_resync:
 			do
 			{
 				++try;
-				if(limit >= 0 && try >= limit) break;
+				if(limit >= 0 && try >= limit) break;				
 
 				if((ret=fr->rd->head_shift(fr,&newhead)) <= 0)
 				{
 					debug("need more?");
 					if(NOQUIET) fprintf (stderr, "Note: Hit end of (available) data during resync.\n");
 
-				goto read_frame_bad;
+					goto read_frame_bad;
 				}
-				debug3("resync try %li at 0x%lx, got newhead 0x%08lx", try, (unsigned long)fr->rd->tell(fr),  newhead);
+				if(VERBOSE3) debug3("resync try %li at 0x%lx, got newhead 0x%08lx", try, (unsigned long)fr->rd->tell(fr),  newhead);
+
 				if(!fr->oldhead)
 				{
 					debug("going to init_resync...");
@@ -614,7 +636,7 @@ init_resync:
 			);
 			/* too many false positives 
 			}while (!(head_check(newhead) && decode_header(fr, newhead))); */
-			if(NOQUIET) fprintf (stderr, "Note: Skipped %li bytes in input.\n", try);
+			if(NOQUIET && fr->silent_resync == 0) fprintf (stderr, "Note: Skipped %li bytes in input.\n", try);
 
 			if(limit >= 0 && try >= limit)
 			{
@@ -673,23 +695,36 @@ init_resync:
 		fr->bsbuf = newbuf;
 	}
 	fr->bsnum = (fr->bsnum + 1) & 1;
+
 	if(!fr->firsthead)
 	{
-		/* In practice, Xing/LAME tags are layer 3 only. */
-		if(fr->lay == 3 && check_lame_tag(fr) == 1)
+		fr->firsthead = newhead; /* _now_ it's time to store it... the first real header */
+		/* This is the first header of our current stream segment.
+		   It is only the actual first header of the whole stream when fr->num is still below zero!
+		   Think of resyncs where firsthead has been reset for format flexibility. */
+		if(fr->num < 0)
 		{
-			if(fr->rd->forget != NULL) fr->rd->forget(fr);
-			fr->oldhead = 0;
-			goto read_again;
+			fr->audio_start = framepos;
+			/* Only check for LAME  tag at beginning of whole stream
+			   ... when there indeed is one in between, it's the user's problem. */
+			if(fr->lay == 3 && check_lame_tag(fr) == 1)
+			{ /* ...in practice, Xing/LAME tags are layer 3 only. */
+				if(fr->rd->forget != NULL) fr->rd->forget(fr);
+
+				fr->oldhead = 0;
+				goto read_again;
+			}
+			/* now adjust volume */
+			do_rva(fr);
 		}
 
-		fr->firsthead = newhead; /* _now_ it's time to store it... the first real header */
-		debug1("fr->firsthead: %08lx", fr->firsthead);
-		/* now adjust volume */
-		do_rva(fr);
+		debug2("fr->firsthead: %08lx, audio_start: %li", fr->firsthead, (long int)fr->audio_start);
 	}
+
   fr->bitindex = 0;
   fr->wordpointer = (unsigned char *) fr->bsbuf;
+	/* Question: How bad does the floating point value get with repeated recomputation?
+	   Also, considering that we can play the file or parts of many times. */
 	if(++fr->mean_frames != 0)
 	{
 		fr->mean_framesize = ((fr->mean_frames-1)*fr->mean_framesize+compute_bpf(fr)) / fr->mean_frames ;
@@ -703,34 +738,25 @@ init_resync:
 		debug("halfspeed - reusing old bsbuf ");
 		memcpy (fr->ssave, fr->bsbuf, fr->ssize);
 	}
+
 	/* index the position */
-	if(INDEX_SIZE > 0 && fr->rdat.flags & READER_SEEKABLE) /* any sane compiler should make a no-brainer out of this */
-	{
-		if(fr->num == fr->index.fill*fr->index.step)
-		{
-			if(fr->index.fill == INDEX_SIZE)
-			{
-				size_t c;
-				/* increase step, reduce fill */
-				fr->index.step *= 2;
-				fr->index.fill /= 2; /* divisable by 2! */
-				for(c = 0; c < fr->index.fill; ++c)
-				{
-					fr->index.data[c] = fr->index.data[2*c];
-				}
-			}
-			if(fr->num == fr->index.fill*fr->index.step)
-			{
-				fr->index.data[fr->index.fill] = framepos;
-				++fr->index.fill;
-			}
-		}
-	}
+#ifdef FRAME_INDEX
+	/* Keep track of true frame positions in our frame index.
+	   but only do so when we are sure that the frame number is accurate... */
+	if(fr->accurate && FI_NEXT(fr->index, fr->num))
+	fi_add(&fr->index, framepos);
+#endif
+
+	if(fr->silent_resync > 0) --fr->silent_resync;
+
 	if(fr->rd->forget != NULL) fr->rd->forget(fr);
+
 	fr->to_decode = fr->to_ignore = TRUE;
 	if(fr->error_protection) fr->crc = getbits(fr, 16); /* skip crc */
+
 	return 1;
 read_frame_bad:
+	fr->silent_resync = 0;
 	if(fr->err == MPG123_OK) fr->err = MPG123_ERR_READER;
 	fr->framesize = oldsize;
 	fr->halfphase = oldphase;
@@ -738,6 +764,62 @@ read_frame_bad:
 }
 
 
+/*
+ * read ahead and find the next MPEG header, to guess framesize
+ * return value: guessed framesize
+ */
+static long guess_freeformat_framesize(mpg123_handle *fr)
+{
+	long i;
+	int ret;
+	unsigned long head;
+	if(!(fr->rdat.flags & (READER_SEEKABLE|READER_BUFFERED)))
+	{
+		if(NOQUIET) error("Cannot look for freeformat frame size with non-seekable and non-buffered stream!");
+		return -1;
+	}
+	/* FIXME: We need proper handling/passing of MPG123_NEED_MORE! */
+	if((ret=fr->rd->head_read(fr,&head))<=0)
+	return -1;
+
+	/* We are already 4 bytes into it */
+	for(i=4;i<65536;i++) {
+		if((ret=fr->rd->head_shift(fr,&head))<=0)
+		{
+			return -1;
+		}
+		if(head_check(head))
+		{
+			int sampling_frequency,mpeg25,lsf;
+			
+			if(head & (1<<20))
+			{
+				lsf = (head & (1<<19)) ? 0x0 : 0x1;
+				mpeg25 = 0;
+			}
+			else
+			{
+				lsf = 1;
+				mpeg25 = 1;
+			}
+			
+			if(mpeg25)
+				sampling_frequency = 6 + ((head>>10)&0x3);
+			else
+				sampling_frequency = ((head>>10)&0x3) + (lsf*3);
+			
+			if((lsf==fr->lsf) && (mpeg25==fr->mpeg25) && (sampling_frequency == fr->sampling_frequency))
+			{
+				fr->rd->back_bytes(fr,i+1);
+				return i-3;
+			}
+		}
+	}
+	fr->rd->back_bytes(fr,i);
+	return -1;
+}
+
+
 /*
  * decode a header and write the information
  * into the frame structure
@@ -793,33 +875,63 @@ static int decode_header(mpg123_handle *fr,unsigned long newhead)
 	fr->copyright = ((newhead>>3)&0x1);
 	fr->original  = ((newhead>>2)&0x1);
 	fr->emphasis  = newhead & 0x3;
+	fr->freeformat = free_format_header(newhead);
 
 	fr->stereo    = (fr->mode == MPG_MD_MONO) ? 1 : 2;
 
 	fr->oldhead = newhead;
-
-	if(!fr->bitrate_index)
+	
+	/* we can't use tabsel_123 for freeformat, so trying to guess framesize... */
+	/* FIXME: We need proper handling/passing of MPG123_NEED_MORE! */
+	if(fr->freeformat)
 	{
-		if(NOQUIET) error1("encountered free format header %08lx in decode_header - not supported yet",newhead);
-
-		return 0;
+		/* when we first encounter the frame with freeformat, guess framesize */
+		if(fr->freeformat_framesize < 0)
+		{
+			fr->framesize = guess_freeformat_framesize(fr);
+			if(fr->framesize > 0)
+			{
+				fr->freeformat_framesize = fr->framesize - fr->padding;
+			}
+			else
+			{
+				error("encountered free format header, but failed to guess framesize");
+				return 0;
+			}
+		}
+		/* freeformat should be CBR, so the same framesize can be used at the 2nd reading or later */
+		else
+		{
+			fr->framesize = fr->freeformat_framesize + fr->padding;
+		}
 	}
 
 	switch(fr->lay)
 	{
+#ifndef NO_LAYER1
 		case 1:
 			fr->do_layer = do_layer1;
-			fr->framesize  = (long) tabsel_123[fr->lsf][0][fr->bitrate_index] * 12000;
-			fr->framesize /= freqs[fr->sampling_frequency];
-			fr->framesize  = ((fr->framesize+fr->padding)<<2)-4;
+			if(!fr->freeformat)
+			{
+				fr->framesize  = (long) tabsel_123[fr->lsf][0][fr->bitrate_index] * 12000;
+				fr->framesize /= freqs[fr->sampling_frequency];
+				fr->framesize  = ((fr->framesize+fr->padding)<<2)-4;
+			}
 		break;
+#endif
+#ifndef NO_LAYER2
 		case 2:
 			fr->do_layer = do_layer2;
-			debug2("bitrate index: %i (%i)", fr->bitrate_index, tabsel_123[fr->lsf][1][fr->bitrate_index] );
-			fr->framesize = (long) tabsel_123[fr->lsf][1][fr->bitrate_index] * 144000;
-			fr->framesize /= freqs[fr->sampling_frequency];
-			fr->framesize += fr->padding - 4;
+			if(!fr->freeformat)
+			{
+				debug2("bitrate index: %i (%i)", fr->bitrate_index, tabsel_123[fr->lsf][1][fr->bitrate_index] );
+				fr->framesize = (long) tabsel_123[fr->lsf][1][fr->bitrate_index] * 144000;
+				fr->framesize /= freqs[fr->sampling_frequency];
+				fr->framesize += fr->padding - 4;
+			}
 		break;
+#endif
+#ifndef NO_LAYER3
 		case 3:
 			fr->do_layer = do_layer3;
 			if(fr->lsf)
@@ -830,12 +942,16 @@ static int decode_header(mpg123_handle *fr,unsigned long newhead)
 			if(fr->error_protection)
 			fr->ssize += 2;
 
-			fr->framesize  = (long) tabsel_123[fr->lsf][2][fr->bitrate_index] * 144000;
-			fr->framesize /= freqs[fr->sampling_frequency]<<(fr->lsf);
-			fr->framesize = fr->framesize + fr->padding - 4;
-		break; 
+			if(!fr->freeformat)
+			{
+				fr->framesize  = (long) tabsel_123[fr->lsf][2][fr->bitrate_index] * 144000;
+				fr->framesize /= freqs[fr->sampling_frequency]<<(fr->lsf);
+				fr->framesize = fr->framesize + fr->padding - 4;
+			}
+		break;
+#endif 
 		default:
-			if(NOQUIET) error("unknown layer type (!!)"); 
+			if(NOQUIET) error1("Layer type %i not supported in this build!", fr->lay); 
 
 			return 0;
 	}
@@ -970,34 +1086,9 @@ int get_songlen(mpg123_handle *fr,int no)
 	if(no < 0) {
 		if(!fr->rd || fr->rdat.filelen < 0)
 			return 0;
-		no = (double) fr->rdat.filelen / compute_bpf(fr);
+		no = (int) ((double) fr->rdat.filelen / compute_bpf(fr));
 	}
 
 	tpf = mpg123_tpf(fr);
-	return no*tpf;
-}
-
-
-
-/* take into account: channels, bytes per sample -- NOT resampling!*/
-off_t samples_to_bytes(mpg123_handle *fr , off_t s)
-{
-	return s
-#ifdef FLOATOUT
-		* 4
-#else
-		* ((fr->af.encoding & MPG123_ENC_16) ? 2 : 1)
-#endif
-		* fr->af.channels;
-}
-
-off_t bytes_to_samples(mpg123_handle *fr , off_t b)
-{
-	return b
-#ifdef FLOATOUT
-		/ 4
-#else
-		/ ((fr->af.encoding & MPG123_ENC_16) ? 2 : 1)
-#endif
-		/ fr->af.channels;
+	return (int) (no*tpf);
 }
